------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 291 - Self Addressing FIFO
--
------------------------------------------------------------------------------
--
--  File name :       incrementer.vhd
--
--  Description :     Incrementer with bit width set by input generic
--
--  Date - revision : December 11th 2002 - v 1.2
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity incrementer is
generic (
	size 		: integer) ;
port (
	ip 		: in std_logic_vector (size-1 downto 0);	-- input
	op 		: out std_logic_vector(size-1 downto 0)); 	-- output = input + 1
end incrementer;

architecture arch_incrementer of incrementer is


signal c 		: std_logic_vector(size-2 downto 0);
signal int 		: std_logic_vector(size-1 downto 0);

begin
op <= int ;

c <= ('0' & ip(size-3 downto 0)) + 1; 					-- the rest count binary

int(size-1) <= ip(size-2) when c(size-2) = '1' else ip(size-1) ; 	-- 2 MSBs of the incrementer count in Gray code
int(size-2) <= not ip(size-1) when c(size-2) = '1' else ip(size-2) ;

int(size-3 downto 0) <= c(size-3 downto 0);				-- creates carry bit at c(size-2)

end arch_incrementer;

